// ignore_for_file: non_constant_identifier_names

import 'dart:async';

import 'package:galaxeus_lib/galaxeus_lib.dart';
import 'package:telegram_client/tdlib/tdlib_ffi/tdlib.dart';
import 'package:telegram_client/telegram_client.dart';
import "package:telegram_client/telegram_client/scheme/scheme.dart"
    as telegram_client_scheme;
import 'package:telegram_client/telegram_client/telegram_client_type.dart';

class TelegramClientTdlibOption {
  Map<dynamic, dynamic>? clientOption;
  bool is_cli;
  int? clientId;
  Duration? invokeTimeOut;
  String event_invoke;
  String event_update;
  Duration? delayUpdate;
  double timeOutUpdate;
  EventEmitter? eventEmitter;
  bool isAutoGetChat;

  FutureOr<Map<dynamic, dynamic>> Function(String, int, LibTdJson)?
      on_get_invoke_data;
  FutureOr<void> Function(dynamic, LibTdJson)? on_receive_update;
  FutureOr<String> Function(int, LibTdJson)? on_generate_extra_invoke;
  bool isInvokeThrowOnError;
  Duration delayInvoke;
  TelegramClientTdlibOption({
    required this.isAutoGetChat,
    this.clientOption,
    required this.is_cli,
    this.clientId,
    this.invokeTimeOut,
    required this.event_invoke,
    required this.event_update,
    required this.timeOutUpdate,
    required this.delayInvoke,
    required this.delayUpdate,
    required this.on_receive_update,
    required this.isInvokeThrowOnError,
  });
}

class TelegramClientTelegramBotApiOption {
  Map clientOption;
  String event_invoke;
  String event_update;
  TelegramClientTelegramBotApiOption({
    required this.clientOption,
    required this.event_invoke,
    required this.event_update,
  });
}

/// return original data json
class TelegramClientLib {
  late Tdlib tdlib;
  late TelegramBotApi telegramBotApi;
  EventEmitter event_emitter = EventEmitter();

  /// return original data json
  TelegramClientLib({EventEmitter? eventEmitter}) {
    if (eventEmitter != null) {
      event_emitter = eventEmitter;
    }
  }

  /// return original data json
  Future<void> init({
    String? pathTdlib,
    required TelegramClientTdlibOption telegramClientTdlibOption,
    required TelegramClientTelegramBotApiOption
        telegramClientTelegramBotApiOption,
  }) async {
    tdlib = Tdlib(
      pathTdl: pathTdlib,
      clientOption: telegramClientTdlibOption.clientOption,
      clientId: telegramClientTdlibOption.clientId,
      invokeTimeOut: telegramClientTdlibOption.invokeTimeOut,
      event_invoke: telegramClientTdlibOption.event_invoke,
      event_update: telegramClientTdlibOption.event_update,
      delayUpdate: telegramClientTdlibOption.delayUpdate,
      delayInvoke: telegramClientTdlibOption.delayInvoke,
      isAutoGetChat: telegramClientTdlibOption.isAutoGetChat,
      on_generate_extra_invoke:
          telegramClientTdlibOption.on_generate_extra_invoke,
      on_get_invoke_data: telegramClientTdlibOption.on_get_invoke_data,
      on_receive_update: telegramClientTdlibOption.on_receive_update,
      isInvokeThrowOnError: telegramClientTdlibOption.isInvokeThrowOnError,
      eventEmitter: event_emitter,
    );
    telegramBotApi = TelegramBotApi(
      tokenBot: "",
      clientOption: telegramClientTelegramBotApiOption.clientOption,
      eventEmitter: event_emitter,
      alfred: null,
    );
  }
}

/// return original data json
class TelegramClientData {
  int? tdlib_client_id;
  TelegramClientType telegramClientType;
  String telegram_bot_api_token_bot;

  /// return original data json
  TelegramClientData({
    required this.telegram_bot_api_token_bot,
    required this.telegramClientType,
    this.tdlib_client_id,
  });

  /// return original data json
  static TelegramClientData tdlib({
    required int tdlib_client_id,
  }) {
    return TelegramClientData(
      telegram_bot_api_token_bot: "",
      telegramClientType: TelegramClientType.tdlib,
      tdlib_client_id: tdlib_client_id,
    );
  }

  /// return original data json
  static TelegramClientData telegramBotApi({
    required String token_bot,
  }) {
    return TelegramClientData(
      telegram_bot_api_token_bot: token_bot,
      telegramClientType: TelegramClientType.telegam_bot_api,
      tdlib_client_id: 0,
    );
  }
}

/// return original data json
class TelegramClient {
  EventEmitter eventEmitter;
  late TelegramClientLib telegram_client_lib;
  late TelegramClientData telegram_client_data;

  /// return original data json
  TelegramClient({
    required TelegramClientLib telegramClientLib,
    required TelegramClientData telegramClientData,
    required this.eventEmitter,
  }) {
    telegramClientLib.tdlib.event_emitter = eventEmitter;
    telegramClientLib.telegramBotApi.event_emitter = eventEmitter;
    telegram_client_lib = telegramClientLib;
    telegram_client_data = telegramClientData;
    telegram_client_lib.tdlib.on(
        telegram_client_lib.tdlib.event_update, (UpdateTd updateTd) async {});
  }

  Listener on(
    String event_name,
  ) {
    return eventEmitter.on(event_name, null, (ev, context) async {});
  }

  /// return original data json
  FutureOr<telegram_client_scheme.JsonScheme> invoke({
    required Map parameters,
    TelegramClientData? telegramClientData,
    TelegramClientLib? telegramClientLib,
    bool is_form = false,
    String? urlApi,
    String? clientType,
    bool isThrowOnError = true,
    void Function(int, int)? onUploadProgress,
    bool isVoid = false,
    Duration? delayDuration,
    Duration? invokeTimeOut,
    String? extra,
    bool? isAutoGetChat,
    bool? isInvokeThrowOnError,
    FutureOr<String> Function(int, LibTdJson libTdJson)? onGenerateExtraInvoke,
    FutureOr<Map<dynamic, dynamic>> Function(String, int, LibTdJson libTdJson)?
        onGetInvokeData,
  }) async {
    telegramClientData ??= telegram_client_data;
    telegramClientLib ??= telegram_client_lib;
    if (telegramClientData.telegramClientType ==
        TelegramClientType.telegam_bot_api) {
      Map respond = await telegramClientLib.telegramBotApi.invoke(
        parameters["@type"],
        parameters: parameters,
        is_form: is_form,
        urlApi: urlApi,
        clientType: clientType,
        isThrowOnError: isThrowOnError,
        onUploadProgress: onUploadProgress,
      );
      if (respond["@type"] == "error") {
        return telegram_client_scheme.Error(respond);
      }
      return telegram_client_scheme.JsonScheme(respond);
    }
    if ((telegramClientData.telegramClientType == TelegramClientType.tdlib)) {
      Map respond = (await telegramClientLib.tdlib.invoke(
        parameters["@type"],
        parameters: parameters,
        isVoid: isVoid,
        delayDuration: delayDuration,
        invokeTimeOut: invokeTimeOut,
        extra: extra,
        isAutoGetChat: isAutoGetChat,
        isInvokeThrowOnError: isInvokeThrowOnError,
        onGenerateExtraInvoke: onGenerateExtraInvoke,
        onGetInvokeData: onGetInvokeData,
      ));
      if (respond["@type"] == "error") {
        return telegram_client_scheme.Error(respond);
      }
      return telegram_client_scheme.JsonScheme(respond);
    }
    telegram_client_scheme.Error error = telegram_client_scheme.Error.create(
      special_extra: "error",
      code: 500,
      message: "telegram_client_type_not_found",
      description: "",
    );

    if (isThrowOnError) {
      throw error;
    }
    return error;
  }
}
